@@ -13,7 +13,7 @@ def regression_space(
         "AdaBoostRegressor":{
             "param_space": [
                 Integer(low=5, high=100, name='n_estimators', num_samples=num_samples),
-                Real(low=0.001, high=1.0, prior='log', name='learning_rate', num_samples=num_samples)],
+                Real(low=0.001, high=1.0, prior='log-uniform', name='learning_rate', num_samples=num_samples)],
             "x0":
                 [50, 1.0]},
         "ARDRegression":{
@@ -150,7 +150,7 @@ def regression_space(
                     True]},
         "OneClassSVM": {
             "param_space": [
-                Categorical(categories=['linear', 'poly', 'rbf', 'sigmoid', 'precomputed'], name='kernel'),
+                Categorical(categories=['linear', 'poly', 'rbf', 'sigmoid'], name='kernel'),
                 Real(low=0.1, high=0.9, name='nu', num_samples=num_samples),
                 Categorical(categories=[True, False], name='shrinking')],
             "x0":
@@ -185,7 +185,7 @@ def regression_space(
                 ['uniform', 'auto', 10, 1]},
         "RANSACRegressor": {
             "param_space": [
-                Integer(low=10, high=1000, name='max_trials'),
+                Integer(low=10, high=1000, name='max_trials', num_samples=num_samples),
                 Real(low=0.01, high=0.99, name='min_samples', num_samples=num_samples)],
             "x0":
                 [10, 0.01]},
@@ -211,7 +211,7 @@ def regression_space(
                 Integer(low=5, high=200, name='n_estimators', num_samples=num_samples),
                 # Maximum tree depth for base learners
                 #Integer(low=3, high=50, name='max_depth', num_samples=num_samples),
-                Real(low=0.0001, high=0.5, name='learning_rate', prior='log', num_samples=num_samples),
+                Real(low=0.0001, high=0.5, name='learning_rate', prior='log-uniform', num_samples=num_samples),
                 Categorical(categories=['gbtree', 'gblinear', 'dart'], name='booster'),
                 # Minimum loss reduction required to make a further partition on a leaf node of the tree.
                 # Real(low=0.1, high=0.9, name='gamma', num_samples=self.num_samples),
@@ -245,7 +245,7 @@ def regression_space(
                 # number of boosting stages to perform
                 Integer(low=5, high=500, name='n_estimators', num_samples=num_samples),
                 #  shrinks the contribution of each tree
-                Real(low=0.001, high=1.0, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.001, high=1.0, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 # fraction of samples to be used for fitting the individual base learners
                 Real(low=0.1, high=1.0, name='subsample', num_samples=num_samples),
                 Real(low=0.1, high=0.9, name='min_samples_split', num_samples=num_samples),
@@ -257,16 +257,16 @@ def regression_space(
                 # todo, during optimization not working with 'rf'
                 Categorical(categories=['gbdt', 'dart', 'goss'], name='boosting_type'),
                 Integer(low=10, high=200, name='num_leaves', num_samples=num_samples),
-                Real(low=0.0001, high=0.1,  name='learning_rate', prior='log', num_samples=num_samples),
-                Integer(low=20, high=500, name='n_estimators', num_samples=num_samples)],
+                Real(low=0.0001, high=0.1,  name='learning_rate', prior='log-uniform', num_samples=num_samples),
+                Integer(low=20, high=100, name='n_estimators', num_samples=num_samples)],
             "x0":
                 ['gbdt', 31, 0.1, 100]},
         "CatBoostRegressor": {
             "param_space": [
                 # maximum number of trees that can be built
-                Integer(low=500, high=5000, name='iterations', num_samples=num_samples),
+                Integer(low=20, high=100, name='iterations', num_samples=num_samples),
                 # Used for reducing the gradient step.
-                Real(low=0.0001, high=0.5, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.5, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 # Coefficient at the L2 regularization term of the cost function.
                 Real(low=0.5, high=5.0, name='l2_leaf_reg', num_samples=num_samples),
                 # arger the value, the smaller the model size.
@@ -279,7 +279,7 @@ def regression_space(
                 Categorical(categories=['Median', 'Uniform', 'UniformAndQuantiles',
                                             'MaxLogSum', 'MinEntropy', 'GreedyLogSum'], name='feature_border_type')],
             "x0":
-                [1000, 0.01, 3.0, 0.5, 0.5, 32, 'GreedyLogSum']},
+                [50, 0.01, 3.0, 0.5, 0.5, 32, 'GreedyLogSum']},
         "DecisionTreeRegressor": {
             "param_space": [
                 Categorical(["best", "random"], name='splitter'),
@@ -334,7 +334,7 @@ def regression_space(
         "HistGradientBoostingRegressor": {
             "param_space": [
                 # Used for reducing the gradient step.
-                Real(low=0.0001, high=0.9, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.9, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 Integer(low=50, high=500, name='max_iter', num_samples=num_samples),  # maximum number of trees.
                 Integer(low=2, high=100, name='max_depth', num_samples=num_samples),  # maximum number of trees.
                 # maximum number of leaves for each tree
@@ -384,7 +384,7 @@ def regression_space(
                 Integer(low=5, high=100, name='n_estimators', num_samples=num_samples),
                 # Maximum tree depth for base learners
                 Integer(low=3, high=50, name='max_depth', num_samples=num_samples),
-                Real(low=0.0001, high=0.5, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.5, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 # Categorical(categories=['gbtree', 'gblinear', 'dart'], name='booster'),  # todo solve error
                 # Minimum loss reduction required to make a further partition on a leaf node of the tree.
                 Real(low=0.1, high=0.9, name='gamma', num_samples=num_samples),
@@ -415,7 +415,12 @@ def regression_space(
     return spaces
 
 
-def classification_space(num_samples:int, verbosity=0):
+def classification_space(num_samples:int,
+                         verbosity=0,
+                         n_features:int = None):
+
+    ridge_cls = _ridge_classifier(num_samples=num_samples)
+    ridge_cls_cv = _ridge_classifiercv()
 
     spaces = {
         "AdaBoostClassifier": {
@@ -490,7 +495,7 @@ def classification_space(num_samples:int, verbosity=0):
                 # number of boosting stages to perform
                 Integer(low=5, high=500, name='n_estimators', num_samples=num_samples),
                 #  shrinks the contribution of each tree
-                Real(low=0.001, high=1.0, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.001, high=1.0, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 # fraction of samples to be used for fitting the individual base learners
                 Real(low=0.1, high=1.0, name='subsample', num_samples=num_samples),
                 Real(low=0.1, high=0.9, name='min_samples_split', num_samples=num_samples),
@@ -500,7 +505,7 @@ def classification_space(num_samples:int, verbosity=0):
         "HistGradientBoostingClassifier": {
             "param_space": [
                 # Used for reducing the gradient step.
-                Real(low=0.0001, high=0.9, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.9, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 Integer(low=50, high=500, name='max_iter', num_samples=num_samples),  # maximum number of trees.
                 Integer(low=2, high=100, name='max_depth', num_samples=num_samples),  # maximum number of trees.
                 # maximum number of leaves for each tree
@@ -544,7 +549,7 @@ def classification_space(num_samples:int, verbosity=0):
             "param_space": [
                 Categorical(categories=['gbdt', 'dart', 'goss', 'rf'], name='boosting_type'),
                 Integer(low=10, high=200, name='num_leaves', num_samples=num_samples),
-                Real(low=0.0001, high=0.1, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.1, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 Integer(low=10, high=100, name='min_child_samples', num_samples=num_samples),
                 Integer(low=20, high=500, name='n_estimators', num_samples=num_samples)],
             "x0":
@@ -640,20 +645,13 @@ def classification_space(num_samples:int, verbosity=0):
             "x0":
                 [100, 5, 2, 0.2, 'auto']},
         "RidgeClassifier": {
-            "param_space": [
-                Real(low=1.0, high=5.0, name='alpha', num_samples=num_samples),
-                Real(low=1e-4, high=1e-1, name='tol', num_samples=num_samples),
-                Categorical(categories=[True, False], name='normalize'),
-                Categorical(categories=[True, False], name='fit_intercept')],
+            "param_space": ridge_cls.space,
             "x0":
-                [1.0, 1e-3, True, True]},
+                ridge_cls.x0},
         "RidgeClassifierCV": {
-            "param_space": [
-                Categorical(categories=[1e-3, 1e-2, 1e-1, 1], name='alphas'),
-                Categorical(categories=[True, False], name='normalize'),
-                Categorical(categories=[True, False], name='fit_intercept')],
+            "param_space": ridge_cls_cv.space,
             "x0":
-                [1e-3,True, True]},
+                ridge_cls_cv.x0},
         "SGDClassifier": {
             "param_space": [
                 Categorical(categories=['l1', 'l2', 'elasticnet'], name='penalty'),
@@ -677,7 +675,7 @@ def classification_space(num_samples:int, verbosity=0):
                 Integer(low=5, high=50, name='n_estimators', num_samples=num_samples),
                 # Maximum tree depth for base learners
                 Integer(low=3, high=30, name='max_depth', num_samples=num_samples),
-                Real(low=0.0001, high=0.5, prior='log', name='learning_rate', num_samples=num_samples),  #
+                Real(low=0.0001, high=0.5, prior='log-uniform', name='learning_rate', num_samples=num_samples),  #
                 Categorical(categories=['gbtree', 'gblinear', 'dart'], name='booster'),
                 Real(low=0.1, high=0.9, name='gamma', num_samples=num_samples),
                 # Minimum loss reduction required to make a further partition on a leaf node of the tree.
@@ -700,7 +698,7 @@ def classification_space(num_samples:int, verbosity=0):
                 Integer(low=5, high=50, name='n_estimators', num_samples=num_samples),
                 # Maximum tree depth for base learners
                 Integer(low=3, high=30, name='max_depth', num_samples=num_samples),
-                Real(low=0.0001, high=0.5, prior='log', name='learning_rate', num_samples=num_samples),  #
+                Real(low=0.0001, high=0.5, prior='log-uniform', name='learning_rate', num_samples=num_samples),  #
                 Categorical(categories=['gbtree', 'gblinear', 'dart'], name='booster'),
                 Real(low=0.1, high=0.9, name='gamma', num_samples=num_samples),
                 # Minimum loss reduction required to make a further partition on a leaf node of the tree.
@@ -720,9 +718,9 @@ def classification_space(num_samples:int, verbosity=0):
         "CatBoostClassifier": {
             "param_space": [
                 # maximum number of trees that can be built
-                Integer(low=50, high=5000, name='iterations', num_samples=num_samples),
+                Integer(low=20, high=100, name='iterations', num_samples=num_samples),
                 # Used for reducing the gradient step.
-                Real(low=0.0001, high=0.5, prior='log', name='learning_rate', num_samples=num_samples),
+                Real(low=0.0001, high=0.5, prior='log-uniform', name='learning_rate', num_samples=num_samples),
                 # depth
                 # https://stackoverflow.com/q/67299869/5982232
                 Integer(1, 15, name="depth", num_samples=num_samples),
@@ -835,3 +833,37 @@ def regression_models()->list:
 def classification_models()->list:
     """returns availabel classification models as list"""
     return list(classification_space(5,0).keys())
+
+
+class _ridge_classifier(object):
+
+    def __init__(self, num_samples=10):
+        import sklearn
+        self.space = [
+            Real(low=1.0, high=5.0, name='alpha', num_samples=num_samples),
+            Real(low=1e-4, high=1e-1, name='tol', num_samples=num_samples),
+            Categorical(categories=[True, False], name='normalize'),
+            Categorical(categories=[True, False], name='fit_intercept')]
+
+        self.x0 = [1.0, 1e-3, True, True]
+
+        if sklearn.__version__ > "1.0.0":
+            self.space.pop(2)
+            self.x0.pop(2)
+
+
+class _ridge_classifiercv(object):
+
+    def __init__(self):
+        import sklearn
+        self.space = [
+             Categorical(categories=[1e-3, 1e-2, 1e-1, 1], name='alphas'),
+             Categorical(categories=[True, False], name='normalize'),
+             Categorical(categories=[True, False], name='fit_intercept')
+        ]
+
+        self.x0 = [1.0, True, True]
+
+        if sklearn.__version__ > "1.0.0":
+            self.space.pop(1)
+            self.x0.pop(1)
