@@ -1,7 +1,7 @@
 
 from .backend import np, tf, keras
 from ._main import BaseModel
-from ai4water.tf_attributes import ACTIVATION_LAYERS, LAYERS,  tcn
+from ai4water.tf_attributes import ACTIVATION_LAYERS, LAYERS,  tcn, MULTI_INPUT_LAYERS
 from .nn_tools import get_add_call_args, get_call_args
 
 
@@ -82,6 +82,12 @@ class Model(BaseModel):
             raise NotImplementedError
         return self._model.outputs
 
+    @property
+    def output_shape(self)->tuple:
+        if self.category == "ML":
+            raise NotImplementedError
+        return self._model.output_shape
+
     @property
     def trainable_weights(self):
         if self.category == "ML":
@@ -141,6 +147,11 @@ class Model(BaseModel):
     def predict_fn(self):
         return self._model.predict
 
+    def count_params(self):
+        if self.category == "ML":
+            raise NotImplementedError
+        return self._model.count_params()
+
     def _get_dummy_input_shape(self):
         shape = ()
         if self.config['backend'] == 'tensorflow' and self.category == "DL":
@@ -307,9 +318,8 @@ class Model(BaseModel):
                     else:
                         call_args, add_args = get_call_args(lyr_inputs, lyr_cache, call_args, lyr_config['name'])
                         layer_initialized = LAYERS[lyr_name](*args, **lyr_config)
-                        # todo, following conditioning is not good
-                        # for concat layer inputs should be ([a,b,c]) instaed of (a,b,c)
-                        if isinstance(lyr_inputs, list) and lyr_name != "Concatenate":
+                        # for multi-input layers inputs should be ([a,b,c]) instaed of (a,b,c)
+                        if isinstance(lyr_inputs, list) and lyr_name in MULTI_INPUT_LAYERS:
                             layer_outputs = layer_initialized(*call_args, **add_args)
                         else:
                             layer_outputs = layer_initialized(call_args, **add_args)
@@ -320,7 +330,7 @@ class Model(BaseModel):
 
             if named_outs is not None:
 
-                if isinstance(named_outs, list):
+                if isinstance(named_outs, (list, tuple)) or named_outs.__class__.__name__ in ["ListWrapper"]:
                     # this layer is returning more than one output
                     assert len(named_outs) == len(layer_outputs), "Layer {} is expected to return {} " \
                                                                   "outputs but it actually returns " \
@@ -334,7 +344,7 @@ class Model(BaseModel):
             self.update_cache(lyr_cache, lyr_config['name'], layer_outputs)
             first_layer = False
 
-        #layer_outputs = self.maybe_add_output_layer(layer_outputs, lyr_cache)
+            self.jsonize_lyr_config(lyr_config)
 
         inputs = []
         for k, v in lyr_cache.items():
@@ -373,7 +383,9 @@ class Model(BaseModel):
         if self.verbosity > 0:
             k_model.summary()
 
-        self.plot_model(k_model)
+        if self.verbosity >= 0:
+            self.plot_model(k_model)
+
         return k_model
 
     def build(self, input_shape=None):
@@ -401,7 +413,7 @@ class Model(BaseModel):
         else:
             self.build_ml_model()
 
-        if not getattr(self, 'from_check_point', False):
+        if not getattr(self, 'from_check_point', False) and self.verbosity>=0:
             # fit may fail so better to save config before as well. This will be overwritten once the fit is complete
             self.save_config()
 
