@@ -1,9 +1,11 @@
 
 from .interpret import Interpret
 from .visualize import Visualize
-from .utils import ProcessPredictions
+from .utils import LossCurve
+from ._process_predictions import ProcessPredictions
 from .explain import ShapExplainer
 from .explain import LimeExplainer
 from .explain import PermutationImportance
 from .explain import PartialDependencePlot
 from ._info_plots import prediction_distribution_plot
+from ._info_plots import feature_interaction
