@@ -1,4 +1,6 @@
-__all__ = ["ACTIVATION_LAYERS", "ACTIVATION_FNS", "LOSSES", "LAYERS", "OPTIMIZERS", "tcn"]
+
+__all__ = ["ACTIVATION_LAYERS", "ACTIVATION_FNS", "LOSSES",
+           "LAYERS", "OPTIMIZERS", "tcn", "MULTI_INPUT_LAYERS"]
 
 # it is supposed that tf is available
 from .backend import get_attributes, tf
@@ -10,11 +12,11 @@ except ModuleNotFoundError:
 
 LOSSES = {}
 LAYERS = {}
+MULTI_INPUT_LAYERS = {"EALSTM"}
 
 if tcn is not None:
     LAYERS.update({"TCN": tcn.TCN})
 
-
 try:
     import atten_lstm
 except ModuleNotFoundError:
@@ -27,9 +29,9 @@ if atten_lstm is not None:
 
 if tf is not None:
     import ai4water.utils.tf_losses as tf_losses
-    from ai4water.models.tensorflow import NBeats
-    import ai4water.models.tensorflow.attention_layers as attns
-    from ai4water.models.tensorflow import TemporalFusionTransformer
+    from ai4water.models._tensorflow import NBeats
+    import ai4water.models._tensorflow.attention_layers as attns
+    from ai4water.models._tensorflow import TemporalFusionTransformer
 
     keras = tf.keras
     LOSSES.update({
@@ -45,7 +47,7 @@ if tf is not None:
     LAYERS.update({"TFT": TemporalFusionTransformer})
     LAYERS.update(get_attributes(aus=tf.keras, what='layers', case_sensitive=True))
 
-    from .models.tensorflow.private_layers import PrivateLayers
+    from .models._tensorflow.private_layers import PrivateLayers
 
     # add private layers to dictionary
     LAYERS.update(get_attributes(aus=PrivateLayers, what='layers', case_sensitive=True))
