@@ -5,6 +5,7 @@ import matplotlib.pyplot as plt
 import pandas as pd
 
 from ai4water import Model
+from ai4water.datasets import mg_photodegradation
 from ai4water.datasets import busan_beach
 from ai4water.postprocessing.explain import PartialDependencePlot
 
@@ -14,21 +15,32 @@ input_features = data.columns.tolist()[0:-1]
 output_features = data.columns.tolist()[-1:]
 
 
-def test_plot_1d_plots(pdp):
-    pdp.plot_1d("tide_cm")
-    pdp.plot_1d("tide_cm", show_dist_as="grid")
-    pdp.plot_1d("tide_cm", show_dist=False)
-    pdp.plot_1d("tide_cm", show_dist=False, ice=False)
-    pdp.plot_1d("tide_cm", show_dist=False, ice=False, model_expected_value=True)
-    pdp.plot_1d("tide_cm", show_dist=False, ice=False, feature_expected_value=True)
-    pdp.plot_1d("tide_cm", ice_only=True, ice_color="red")
-    pdp.plot_1d("tide_cm", ice_only=True, ice_color="Blues")
+def test_plot_1d_plots(pdp, feature="tide_cm"):
+    pdp.plot_1d(feature)
+    pdp.plot_1d(feature, show_dist_as="grid")
+    pdp.plot_1d(feature, show_dist=False)
+    pdp.plot_1d(feature, show_dist=False, ice=False)
+    pdp.plot_1d(feature, show_dist=False, ice=False, model_expected_value=True)
+    pdp.plot_1d(feature, show_dist=False, ice=False, feature_expected_value=True)
+    pdp.plot_1d(feature, ice_only=True, ice_color="red")
+    pdp.plot_1d(feature, ice_only=True, ice_color="Blues")
 
     return
 
 
 class TestPDP(unittest.TestCase):
 
+    def test_with_cat_data(self):
+        data, cat_enc, an_enc  = mg_photodegradation(encoding="ohe")
+        model = Model(model="XGBRegressor", verbosity=0)
+        model.fit(data=data)
+        x, _ = model.training_data(data=data)
+        pdp = PartialDependencePlot(model.predict, x, model.input_features,
+                                        num_points=14, show=False, save=False)
+        feature = [f for f in model.input_features if f.startswith('Catalyst_type')]
+        test_plot_1d_plots(pdp, feature)
+        return
+
     def test_2d_data_single_input(self):
         model = Model(model="XGBRegressor",
                       verbosity=0)
@@ -73,12 +85,12 @@ class TestPDP(unittest.TestCase):
                                     num_points=14, show=False, save=False)
 
         pdp.nd_interactions(show_dist=True)
-
-        ax = pdp.plot_interaction(["tide_cm", "wat_temp_c"])
+        #
+        ax = pdp.plot_interaction(["tide_cm", "wat_temp_c"], show=False, save=False)
         assert isinstance(ax, plt.Axes)
 
         model.partial_dependence_plot(x=pd.DataFrame(x, columns=model.input_features),
-                                      feature_name='tide_cm')
+                                     feature_name='tide_cm', show=False)
 
         return
 
